import NetInfo from "@react-native-community/netinfo";
import { PureJSCrypto } from "cojson/dist/crypto/PureJSCrypto"; // Importing from dist to not rely on the exports field
import { createInviteLink as baseCreateInviteLink, createAnonymousJazzContext, createJazzContext, } from "jazz-tools";
import { KvStoreContext } from "./storage/kv-store-context.js";
import { SQLiteReactNative } from "./storage/sqlite-react-native.js";
import { WebSocketPeerWithReconnection } from "cojson-transport-ws";
class ReactNativeWebSocketPeerWithReconnection extends WebSocketPeerWithReconnection {
    onNetworkChange(callback) {
        return NetInfo.addEventListener((state) => callback(state.isConnected ?? false));
    }
}
async function setupPeers(options) {
    const CryptoProvider = options.CryptoProvider || PureJSCrypto;
    const crypto = await CryptoProvider.create();
    let node = undefined;
    const peersToLoadFrom = [];
    if (options.storage && options.storage !== "disabled") {
        const storage = await SQLiteReactNative.asPeer({
            adapter: options.storage,
        });
        peersToLoadFrom.push(storage);
    }
    if (options.sync.when === "never") {
        return {
            toggleNetwork: () => { },
            peersToLoadFrom,
            setNode: () => { },
            crypto,
        };
    }
    const wsPeer = new ReactNativeWebSocketPeerWithReconnection({
        peer: options.sync.peer,
        reconnectionTimeout: options.reconnectionTimeout,
        addPeer: (peer) => {
            if (node) {
                node.syncManager.addPeer(peer);
            }
            else {
                peersToLoadFrom.push(peer);
            }
        },
        removePeer: (peer) => {
            peersToLoadFrom.splice(peersToLoadFrom.indexOf(peer), 1);
        },
    });
    function toggleNetwork(enabled) {
        if (enabled) {
            wsPeer.enable();
        }
        else {
            wsPeer.disable();
        }
    }
    function setNode(value) {
        node = value;
    }
    if (options.sync.when === "always" || !options.sync.when) {
        toggleNetwork(true);
    }
    return {
        toggleNetwork,
        peersToLoadFrom,
        setNode,
        crypto,
    };
}
export async function createJazzReactNativeGuestContext(options) {
    const { toggleNetwork, peersToLoadFrom, setNode, crypto } = await setupPeers(options);
    const context = await createAnonymousJazzContext({
        crypto,
        peersToLoadFrom,
    });
    setNode(context.agent.node);
    options.authSecretStorage.emitUpdate(null);
    return {
        guest: context.agent,
        node: context.agent.node,
        done: () => {
            // TODO: Sync all the covalues before closing the connection & context
            toggleNetwork(false);
            context.done();
        },
        logOut: () => {
            return context.logOut();
        },
    };
}
export async function createJazzReactNativeContext(options) {
    const { toggleNetwork, peersToLoadFrom, setNode, crypto } = await setupPeers(options);
    let unsubscribeAuthUpdate = () => { };
    if (options.sync.when === "signedUp") {
        const authSecretStorage = options.authSecretStorage;
        const credentials = options.credentials ?? (await authSecretStorage.get());
        // To update the internal state with the current credentials
        authSecretStorage.emitUpdate(credentials);
        function handleAuthUpdate(isAuthenticated) {
            if (isAuthenticated) {
                toggleNetwork(true);
            }
            else {
                toggleNetwork(false);
            }
        }
        unsubscribeAuthUpdate = authSecretStorage.onUpdate(handleAuthUpdate);
        handleAuthUpdate(authSecretStorage.isAuthenticated);
    }
    const context = await createJazzContext({
        credentials: options.credentials,
        newAccountProps: options.newAccountProps,
        peersToLoadFrom,
        crypto,
        defaultProfileName: options.defaultProfileName,
        AccountSchema: options.AccountSchema,
        sessionProvider: provideLockSession,
        authSecretStorage: options.authSecretStorage,
    });
    setNode(context.node);
    return {
        me: context.account,
        node: context.node,
        authSecretStorage: context.authSecretStorage,
        done: () => {
            // TODO: Sync all the covalues before closing the connection & context
            toggleNetwork(false);
            unsubscribeAuthUpdate();
            context.done();
        },
        logOut: () => {
            unsubscribeAuthUpdate();
            return context.logOut();
        },
    };
}
export async function provideLockSession(accountID, crypto) {
    const sessionDone = () => { };
    const kvStore = KvStoreContext.getInstance().getStorage();
    const sessionID = (await kvStore.get(accountID)) ||
        crypto.newRandomSessionID(accountID);
    await kvStore.set(accountID, sessionID);
    return Promise.resolve({
        sessionID,
        sessionDone,
    });
}
/** @category Invite Links */
export function createInviteLink(value, role, { baseURL, valueHint } = {}) {
    return baseCreateInviteLink(value, role, baseURL ?? "", valueHint);
}
export function setupKvStore(kvStore) {
    if (!kvStore) {
        return undefined;
    }
    KvStoreContext.getInstance().initialize(kvStore);
    return kvStore;
}
//# sourceMappingURL=platform.js.map