export class SQLiteClient {
    adapter;
    initializationPromise = null;
    isInitialized = false;
    constructor(adapter, _) {
        this.adapter = adapter;
    }
    async initializeInternal() {
        try {
            await this.adapter.initialize();
            this.isInitialized = true;
        }
        catch (error) {
            console.error("[SQLiteClient] initialization failed:", error);
            this.initializationPromise = null;
            throw error;
        }
    }
    async ensureInitialized() {
        if (this.isInitialized) {
            return;
        }
        if (!this.initializationPromise) {
            this.initializationPromise = this.initializeInternal();
        }
        await this.initializationPromise;
    }
    async getCoValue(coValueId) {
        await this.ensureInitialized();
        try {
            const { rows } = await this.adapter.execute("SELECT * FROM coValues WHERE id = ?", [coValueId]);
            if (!rows || rows.length === 0)
                return;
            const coValueRow = rows[0];
            const parsedHeader = coValueRow?.header &&
                JSON.parse(coValueRow.header);
            return {
                ...coValueRow,
                header: parsedHeader,
            };
        }
        catch (e) {
            console.warn("[SQLiteClient] Error getting coValue:", coValueId, e);
            return;
        }
    }
    async getCoValueSessions(coValueRowId) {
        await this.ensureInitialized();
        const { rows } = await this.adapter.execute("SELECT * FROM sessions WHERE coValue = ?", [coValueRowId]);
        return rows;
    }
    async getSingleCoValueSession(coValueRowId, sessionID) {
        await this.ensureInitialized();
        const { rows } = await this.adapter.execute("SELECT * FROM sessions WHERE coValue = ? AND sessionID = ?", [coValueRowId, sessionID]);
        return rows[0];
    }
    async getNewTransactionInSession(sessionRowId, firstNewTxIdx) {
        const { rows } = await this.adapter.execute("SELECT * FROM transactions WHERE ses = ? AND idx >= ?", [sessionRowId, firstNewTxIdx]);
        if (!rows || rows.length === 0)
            return [];
        try {
            return rows.map((row) => {
                const rowData = row;
                return {
                    ...rowData,
                    tx: JSON.parse(rowData.tx),
                };
            });
        }
        catch (e) {
            console.warn("Invalid JSON in transaction", e);
            return [];
        }
    }
    async getSignatures(sessionRowId, firstNewTxIdx) {
        const { rows } = await this.adapter.execute("SELECT * FROM signatureAfter WHERE ses = ? AND idx >= ?", [sessionRowId, firstNewTxIdx]);
        return rows;
    }
    async addCoValue(msg) {
        await this.ensureInitialized();
        const { insertId } = await this.adapter.execute("INSERT INTO coValues (id, header) VALUES (?, ?)", [msg.id, JSON.stringify(msg.header)]);
        return insertId ?? 0;
    }
    async addSessionUpdate({ sessionUpdate, }) {
        await this.ensureInitialized();
        const { rows } = await this.adapter.execute(`INSERT INTO sessions (coValue, sessionID, lastIdx, lastSignature, bytesSinceLastSignature)
       VALUES (?, ?, ?, ?, ?)
       ON CONFLICT(coValue, sessionID)
       DO UPDATE SET lastIdx=excluded.lastIdx,
                    lastSignature=excluded.lastSignature,
                    bytesSinceLastSignature=excluded.bytesSinceLastSignature
       RETURNING rowID`, [
            sessionUpdate.coValue,
            sessionUpdate.sessionID,
            sessionUpdate.lastIdx,
            sessionUpdate.lastSignature,
            sessionUpdate.bytesSinceLastSignature ?? 0,
        ]);
        return rows[0]?.rowID;
    }
    async addTransaction(sessionRowID, nextIdx, newTransaction) {
        await this.ensureInitialized();
        const { rowsAffected } = await this.adapter.execute("INSERT INTO transactions (ses, idx, tx) VALUES (?, ?, ?)", [sessionRowID, nextIdx, JSON.stringify(newTransaction)]);
        return rowsAffected;
    }
    async addSignatureAfter({ sessionRowID, idx, signature, }) {
        await this.ensureInitialized();
        const { rowsAffected } = await this.adapter.execute("INSERT INTO signatureAfter (ses, idx, signature) VALUES (?, ?, ?)", [sessionRowID, idx, signature]);
        return rowsAffected;
    }
    async transaction(operationsCallback) {
        try {
            await this.ensureInitialized();
            await this.adapter.transaction(async () => {
                await operationsCallback();
            });
        }
        catch (e) {
            console.error("Transaction failed:", e);
            throw e;
        }
    }
}
//# sourceMappingURL=client.js.map